angular.module('hh')
.service('ImageResizer', ['$q', function($q) {
	/**
	 * @typedef Dimension
	 * @type {Object}
	 * @property {number} width target width
	 * @property {number} height target height
	 */

	/**
	 * @typedef CropOptions
	 * @type {Object}
	 * @property {number} width target width
	 * @property {number} height target height
	 * @property {number} x x coordinate
	 * @property {number} y y coordinate
	 */

	function _image(string, done) {
		var image = new Image();
    image.onerror = function(e) {
      console.error('TODO: Something went wrong processing the image.');
    };

		image.onload = function() {
			done(image);
		};
		image.src = string;
	}

	function _canvas(image, dimensions) {
		if (!(image instanceof Node)) {
			throw new TypeError('imageResizer#canvas requires an Image node');
		}
		if (typeof dimensions === 'undefined') {
			// cheat since we only care about the width/height so we’ll just alias
			// dimensions to get those props
			dimensions = image;
		}

		var canvas = document.createElement('canvas');
		canvas.width = dimensions.width;
		canvas.height = dimensions.height;
		canvas
			.getContext('2d')
			.drawImage(image, 0, 0, image.width, image.height);

		return canvas;
	}

	/**
	 * Helper function to resolve an image and return a function.
	 * *Should* be used by all processing functions.
	 *
	 * @param {ImageElement|String} image image to process
	 * @param {Object} options options hash
	 * @param {Function} fn the processing function
	 */
	function _process(image, options, fn) {
		var	deferred = $q.defer();

		// if `image` is a base64 string
		if (typeof image === 'string') {
			_image(image, function(img) {
				fn(img, options, deferred);
			});
		}
		else fn(image, options, deferred);

		return deferred.promise;
	}

	/**
	 * Crop an image.
	 *
	 * @param {ImageElement|String} image image to process
	 * @param {CropOptions} options options
	 * @return {Promise} $q.deferred promise object
	 */
	function crop(image, options) {
		return _process(image, options, function(image, options, deferred) {
			var from = _canvas(image);
			if (from.width === options.width && from.height === options.height) {
				return deferred.resolve(from.toDataURL('image/jpeg'));
			}

			var c = document.createElement('canvas');
			c.width = options.width;
			c.height = options.height;
			var x = center('x', options, image);
			var y = center('y', options, image);


			// TODO: default crop to center
			c.getContext('2d').drawImage(
				image,
				x, y,
				image.width, image.height,
				0, 0,
				options.width, options.height
			);

			deferred.resolve(c.toDataURL('image/jpeg'));
		});
	}

	/**
	 * Resize an image.
	 * Constrains dimensions to aspect-ratio weighting
	 * the width as the primary dimension.
	 *
	 * @param {ImageElement|String} image image to process
	 * @param {Dimension} dimensions options hash
	 * @return Promise} $q.deferred promise object
	 */
	function resize(image, dimensions) {
		return _process(image, dimensions, function(image, dimensions, deferred) {
			dimensions = constrainToAspectRatio(image, dimensions);

			var from = _canvas(image);
			var to = _canvas(image, dimensions);

			pica.resizeCanvas(
				from,
				to,
				{transferable: true},
				function(err) {
					if (err) deferred.reject(err);
					else deferred.resolve(to.toDataURL('image/jpeg'));
				}
			);
		});
	}

	/*
	console.log(constrainToAspectRatio(
		{width: 1000, height: 1500},
		{width: 200, height: 200}
	));

	console.log(constrainToAspectRatio(
		{width: 1500, height: 1000},
		{width: 200, height: 200}
	));
	*/

	// resets dimensions twithrespect to
	// aspect-ratio of original image
	function constrainToAspectRatio(image, dimensions) {
		var origWidth = image.width;
		var origHeight = image.height;
		var ratio = origWidth / origHeight;
		if (origWidth > origHeight) {
			dimensions.height = Math.floor(origHeight * dimensions.width / origWidth);
		}
		else {
			dimensions.width = Math.floor(origWidth * dimensions.height / origHeight);
		}

		return dimensions;
	}

	// get defined or center coordinate of an image
	function center(coord, options, image) {
		if (typeof options[coord] === 'number') {
			return options[coord];
		}

		var dimension = coord === 'x' ? 'width' : 'height';
		return Math.abs(image[dimension] - options[dimension]) / 2;
	}

	return {
		resize: resize,
		crop: crop,
	};

}]);
