'use strict';
/**
 * @ngdoc service
 * @name level.service:LocalStorageSrvc
 * @requires $timeout
 * @requires $window
 *
 * @description
 * A centralized service wrapper for accessing and manipulating local
 * storage in the browser. Sets all values as JSON and returns the
 * JSON parsed value for all keys when retrieved.
 */

angular.module('hh.LocalStorageService', []);

angular.module('hh.LocalStorageService').factory('LocalStorageService', [
		'$timeout',
		'$window',
	function LocalStorageServiceFactory(
	    $timeout,
	    $window
	) {
		// Default to in-memory object store as fallback(basically a safe guard for private browsing in Safari)
		var dataStore = {
			data: {},
			getItem: function (key) {
				return dataStore.data[key];
			},
			removeItem: function (key) {
				delete dataStore.data[key];
			},
			setItem: function (key, value) {
				dataStore.data[key] = value;
			}
		};

		/**
		* Initializes the local storage service using the appropriate
		* storage type (local or session).
		*
		* @method init
		* @param {Boolean} [persist] Use browser localStorage (persistant across sessions)
		* instead of sessionStorage.
		*/
		function init(persist) {
			if (!Modernizr.localstorage || !Modernizr.sessionstorage) {
				// Display unsupported browser error message
				console.log('localStorage and/or sessionStorage are unsupported, using local object as fallback');

				// User is probably in private browsing mode...
				if (/safari/gi.test(navigator.userAgent)) {
					// trigger on a timeout to ensure the listeners are set
					/*$timeout(function () {
						$rootScope.$broadcast('NoticeManagerSrvc.requestShow',
							{
								id: 'localstorageNotSupported',
								location: 'global',
								type: 'error',
								message: $i18next('notifications.error.private_browsing'),
								dismissible: {
									user: true
								}
							}
						);
					}, 300);*/
				}
				return;
			}

			// default to persistent if there is currently user data in localStorage
			dataStore = (persist || $window.localStorage.getItem('user'))?
				$window.localStorage : $window.sessionStorage;
		}

		/**
		* Clears all local storage (local and session).
		*
		* @method clear
		*/
		function clear() {
			// clear both localStorage and sessionStorage for consistency
			try {
				$window.localStorage.clear();
				$window.sessionStorage.clear();

				if (dataStore.data) {
					dataStore.data = {};
				}
			}
			catch (e) {
				console.error('[LocalStorageSrvc.clear()]', e);
			}
		}

		/**
		* Get JSON parsed value from local storage.
		*
		* @method getItem
		* @param {String} key The key of the value to be retrieved.
		* @return {Object} The JSON parsed result of the value found at
		*   the given key.
		*/
		function getItem(key) {
			return angular.fromJson(dataStore.getItem(key));
		}

		/**
		* Remove a value from local storage.
		*
		* @method removeItem
		* @param {String} key The key of the value to be removed.
		*/
		function removeItem(key) {
			try {
				dataStore.removeItem(key);
			}
			catch (e) {
				console.log('LocalStorage removeItem error');
			}
		}

		/**
		* Save a JSON stringified value to local storage.
		*
		* @method setItem
		* @param {String} key The key of the value to be set.
		* @param {Object} value The object to store as the value in JSON.
		*/
		function setItem(key, value) {
			/* There is no way to know if you exceeded the storage
			* cross-browser, so wrap your store commands in try/catch.
			* Up to 2.5MB is safe to use. (http://html5please.com/#localStorage)
			*/
			try {
				dataStore.setItem(key, angular.toJson(value));
			}
			catch (e) {
			   console.log('LocalStorage setItem error');
			}
		}


		// init
		init();

		return {
			clear: clear,
			init: init,
			getItem: getItem,
			removeItem: removeItem,
			setItem: setItem
		};
	}
]);
