'use strict';

angular.module('hh.UserService', []).service('UserService', [
	'$http',
	'$q',
	'$rootScope',
	'$ionicModal',
	'AppConfig',
	'LocalStorageService',
function(
	$http,
	$q,
	$rootScope,
	$ionicModal,
	AppConfig,
	LocalStorageService
) {
	var followingCache;
	var userModel;
	
	// private functions

	function login(serviceName, scope, deferred) {
		hello(serviceName).login(scope).then(function(res) {
			console.log("UserService.login", res);
			if (res.error) {
				var error = '';
				if (res.error.code === 'cancelled') {
					error = res.error.code;
				}
				else {
					error = 'error logging to ' + serviceName + ' .';
				}

				deferred.reject(error);
				return;
			}

			if (res.authResponse) {
				deferred.resolve(res.authResponse);
			}
		},
		function(error) {
			// console.log("UserService.login.error", error);
			deferred.reject(error);
		});
	}

	function setUserModel(data) {
		// set user model
		if (userModel)
			userModel = angular.extend(userModel, data);
		else
			userModel = data;

		LocalStorageService.setItem('session', userModel);

		// broadcast change to listeners
		$rootScope.$broadcast('UserService.onUserModelChange', { userModel: userModel });
	}


	// public functions

	this.clearUserModel = function() {
		// set data
		userModel = null;
		
		LocalStorageService.removeItem('session');
	}

	// api with callback
	this.callAPI = function(route, params, callback, isOpen) {
		var url = AppConfig.urlAPI + route + (isOpen ? '' : userModel.userId + '/' + userModel.session);
		$http.post(url, params)
			.success(function(data, status) {
				if (callback) {
					callback(data);
				}
			})
			.error(function(data, status) {
				console.log('Error:' + data + '\n Status:' + status);
			});
	};
	
	// api with promise
	this.postAPI = function(route, params, isOpen) {
		var deferred = $q.defer();

		$http.post(AppConfig.urlAPI + route + (isOpen ? '' : userModel.userId + '/' + userModel.session), params)
			.success(function(res) {
				deferred.resolve(res);
			})
			.error(function(res) {
				deferred.reject(res);
			});
		return deferred.promise;
	};

	// follow / followers
	this.getFollowers = function(userId, withProfile) {
		var deferred = $q.defer();

		$http.post(AppConfig.urlAPI + '/followers/get/' + userId + '/' + userModel.session, { userId: userId, profile: withProfile })
			.success(function(res) {
				deferred.resolve(res);
			})
			.error(function(res) {
				deferred.reject(res);
			});
		return deferred.promise;
	};

	this.getMyFollowers = function(withProfile) {
		return this.getFollowers(userModel.userId, withProfile);
	};

	this.getFollowings = function(userId, withProfile) {
		var deferred = $q.defer();
		$http.post(AppConfig.urlAPI + '/followers/get/' + userId + '/' + userModel.session, { followerId: userId, profile: withProfile })
		.success(function(res) {
			deferred.resolve(res);
		})
		.error(function(res) {
			deferred.reject(res);
		});
		return deferred.promise;
	};

	this.getMyFollowings = function(withProfile) {
		return this.getFollowings(userModel.userId, withProfile);
	};

	this.isFollowing = function(userId) {
		var deferred = $q.defer();

		if (followingCache) {
			deferred.resolve(_.contains(followingCache, userId));
		}
		else {
			this.getMyFollowings().then(
				function(followings) {
					followingCache = _.pluck(followings, 'userId');
					deferred.resolve(_.contains(followingCache, userId));
				},
				function(err) {
					deferred.reject(err);
				}
			);
		}

		return deferred.promise;
	};

	this.followUser = function(userId) {
		var deferred = $q.defer();

		if (!userModel.session || !userModel.userId) {
			deferred.reject('must be logged in to follow');
		}
		else {
			$http.post(AppConfig.urlAPI + '/follower/add/' + userModel.userId + '/' + userModel.session, { followingId: userId })
			.success(function(res) {
				if (followingCache) {
					followingCache = _.union(followingCache, [userId]);
				}

				deferred.resolve(res);
			})
			.error(function(res) {
				deferred.reject(res);
			});
		}

		return deferred.promise;
	};

	this.unfollowUser = function(userId) {
		var deferred = $q.defer();

		if (!userModel.session || !userModel.userId) {
			deferred.reject('must be logged in to unfollow');
		}
		else {
			$http.post(AppConfig.urlAPI + '/follower/delete/' + userModel.userId + '/' + userModel.session, {followingId: userId})
			.success(function(res) {
				if (followingCache) {
					followingCache = _.without(followingCache, userId);
				}

				deferred.resolve(res);
			});
		}

		return deferred.promise;
	};

	this.isLoggedIn = function() {
		return !!userModel;
	};

	this.getLoggedInUser = function getLoggedInUser() {
		var userModel;
		try {
			userModel = JSON.parse(window.localStorage.getItem('session'));
		}
		catch (error) {
			return null;
		}

		if (!userModel || !userModel.userId || !userModel.session) {
			return null;
		}

		/*
		//implement promise for verify session
		this.verifySession(function(data, status) {
			if (data && data.userId) {
				UserModel = data;
			}
		});
		*/
		return userModel;
	};

	this.logout = function logout() {
		this.clearUserModel();
	};

	//@params -> {userId: STRING, session: STRING}
	this.verifySession = function(callback) {
		this.callAPI('/user/verifysession/', null, callback);
	};

	// get self
	this.getSelf = function() {
		return this.postAPI('/user/get/');
	};

	// get user by params
	this.getUser = function(params) {
		return this.postAPI('/user/get/', params);
	};
	
	// status whether user's geo / local time is restricted
	this.getUserStatus = function() {
		var params = { latitude: userModel.latitude, longitude: userModel.longitude, appVersion: _appVersion };
		return this.postAPI('/user/status/', params);
	};

	this.createSession = function(params, callback) {
		console.log('creating session');
		var deferred = $q.defer();
		var self = this;

		$http.post(AppConfig.urlAPI + '/user/connect/', params)
			.success(function(data, status) {
				console.log('created session - args: ', arguments);
				setUserModel(data.data);

				deferred.resolve(data);
			})
			.error(function(data, status) {
				console.log('Error:' + data + '\n Status:' + status);
				deferred.reject(data);
			});

		return deferred.promise;
	};

	this.update = function(params, callback) {
		setUserModel(params);
		this.callAPI('/user/update/', params, callback);
	};

	this.flag = function(params, callback) {
		this.callAPI('/user/flag/', params, callback);
	};

	this.block = function(params, callback) {
		this.callAPI('/user/block/', params, callback);
	};

	this.phoneUser = function(params, callback) {
		this.callAPI('/user/phone/', params, callback);
	};

	this.confirmUser = function(params, callback) {
		this.callAPI('/user/confirm/', params, callback);
	};

	this.confirmedUser = function(callback) {
		this.callAPI('/user/confirmed/', null, callback);
	};
	
	this.addFriend = function(params, callback) {
		var friend = { phoneNumber: params.phoneNumber };
		if (params.name)
			friend.name = params.name;
		this.callAPI('/friends/add/', friend, callback);
	};
	
	this.inviteFriend = function(params, callback) {
		var friend = { phoneNumber: params.phoneNumber };
		this.callAPI('/friends/invite/', friend, callback);
	};
	
	// get mutual friends
	this.getMutualFriends = function(params) {
		return this.postAPI('/friends/mutual/', params);
	};
	
	
	// get messages
	this.getMessages = function(params) {
		return this.postAPI('/messages/get/', params);
	};

	// OAuth
	this.oauth = function(serviceName, scope) {
		var deferred = $q.defer();

		login(serviceName, scope, deferred);

		return deferred.promise;
	};
	
	// sync user model
	this.syncUserModel = function(scopedUserModel, callback) {
		this.getSelf()
		.then(
			function(response) {
				setUserModel(response.data);
				scopedUserModel = userModel;
				if (callback) {
					callback(userModel);
				}
			}
		);
	};
	
	this.setUserModel = setUserModel;

	// init
	userModel = this.getLoggedInUser();
}
]);
